#ifndef CPPE_COMMENT_FILTER_HPP
#define CPPE_COMMENT_FILTER_HPP

#include <string>
#include <vector>

#include <cppe/string_helpers.hpp>
#include <cppe/cppe_exception.hpp>

namespace cppe { 

/**@brief
 * The comment_filter class makes it easy to seperate
 * comments from the relevant text. Only one-line comments are
 * supported, but you can have as many comment starters
 * as you like (e.g #, //, ;, ! and so on).
 */

class comment_filter {
	public:

	comment_filter() {}
	
	/**@brief 
	 * Used to add comment starters to the list.
	 * @post The comment starter is added to the list.
	 * @param comment_starter The string that defindes the start of a comment.
	 */						
	void add_comment_starter(const std::string &comment_starter) {
		_comment_starters.push_back(comment_starter);
	}

	/**@brief 
	 * Find the start of a comment.
	 * @param line The string that should be searched for comments.
	 * @return Where the start of the comment is at.
	 */
	std::string::size_type find_comment(const std::string &line) const;

	/**@brief 
	 * Find the start of a comment.
	 * @param line The string that should be searched for comments.
	 * @param comment The comment starter found.
	 * @return Where the start of the comment is at.
	 */
	std::string::size_type find_comment(const std::string &line, std::string &comment) const;
	
	private:
	/**A list of all the comment starters the filter will use */
	std::vector<std::string> _comment_starters;
};

} //End namespace cppe

#endif

