#ifndef CPPE_FIELD_HPP
#define CPPE_FIELD_HPP

#include <vector>
#include <string>

namespace cppe {

class field {
	public:
	typedef std::string::size_type ssize;

	field(const std::string &target) : _target(target), _begin(0), _size(target.size()), _end(target.size()-1) {}
	field(const std::string &target, ssize from, ssize size) : _target(target), _begin(from), _size(size), _end(from+size-1) {}

	unsigned int fields() const { return _subfields.size(); }
	field& operator[](unsigned int i) { return *_subfields[i]; }

	bool empty() const {
		if(_size == 0)
			return true;
		return false;
	}

	ssize size() const { return _size; }

	ssize begin() const { return _begin; }

	ssize end() const { return _end; }

	std::string str() const { return _target.substr(_begin, _size); }

	char at(ssize index) { return _target.at(_begin + index); }

	bool compare(const std::string &cmp) const {
		if(_target.compare(_begin, _size, cmp) == 0)
			return true;
		return false;
	}

	void extend(unsigned int from) { extend(from, _subfields.size()-1); }
	void extend(unsigned int from, unsigned int too);

	void split();

	void split(char separator);

	void clear() {
		for(unsigned int i = 0; i < _subfields.size(); ++i)
			delete _subfields[i];
		_subfields.clear();
	}

	~field() { clear(); }

	private:
	friend bool operator==(const field &f, const field &f2);
	const std::string &_target;
	ssize _begin;
	ssize _size;
	ssize _end;
	std::vector<field*> _subfields;
};

inline bool operator==(const std::string &cmp, const field &f) {
	return f.compare(cmp);
}

inline bool operator==(const field &f, const std::string &cmp) {
	return f.compare(cmp);
}

inline bool operator!=(const std::string &cmp, const field &f) {
	return !f.compare(cmp);
}

inline bool operator!=(const field &f, const std::string &cmp) {
	return !f.compare(cmp);
}

inline bool operator==(const field &f, const field &f2) {
	if(f._target.compare(f._begin, f._size, f2._target, f2._begin, f2._size) == 0)
		return true;
	return false;
}

inline bool operator!=(const field &f, const field &f2) {
	if(f == f2)
		return false;
	return true;
}

} //End namespace cppe

#endif
