#ifndef CPPE_FILTER_ITERATOR_HPP
#define CPPE_FILTER_ITERATOR_HPP

#include <string>

#include <cppe/comment_filter.hpp>
#include <cppe/string_helpers.hpp>

namespace cppe {

class filter_iterator {
	public:
	typedef std::string::size_type ssize;
	filter_iterator(const comment_filter &filter) : 
				_line(0),
				_first_non_ws(std::string::npos),
				_last_non_ws(std::string::npos),
				_filter(filter) {}
	unsigned int line() const { return _line; }
	ssize first() const { return _first_non_ws; }
	ssize last() const { return _last_non_ws; }

	template<typename BEGIN, typename END> bool next(BEGIN &from, const END &end) {
		for(; from != end; ++from) {
			++_line;
			std::string::size_type first = find_non_ws(*from);
			if(first == std::string::npos) //empty string
				continue;
			std::string::size_type comment = _filter.find_comment(*from);
			if(first == comment)	//It is a comment
				continue;
		
			if(comment != std::string::npos)
				_last_non_ws = rfind_non_ws(*from, --comment);
			else
				_last_non_ws = rfind_non_ws(*from);
			_first_non_ws = first;
			break;
		}
		if(from != end)
			return true;
		return false;
	}

	private:
	unsigned int _line;
	ssize _first_non_ws;
	ssize _last_non_ws;
	const comment_filter &_filter;
};

} //End namespace cppe

#endif
