#ifndef CPPE_FLAGS_HPP
#define CPPE_FLAGS_HPP

#include <string>

#include <sys/stat.h>	//u_int32_t

#include <cppe/cppe_exception.hpp>

namespace cppe { 

/**@brief
 * The flags class provide easy handling of filsystem flags.
 */

class flags {
	public:
	class bad_flag_exception : public cppe_exception {
		public:
		bad_flag_exception(const std::string& m) : cppe_exception(m) {}
	};
	
	/**@brief 
	 * Constructor.
	 * @param f A comma seperated list of flags.
	 * @throws bad_flag_exception if \c f has an unknown flag.
	 */
	//flags(const std::string &f);
	flags(u_int32_t f) : _flags(f) {}
	
	operator u_int32_t() const { return _flags; }
	
	/**@brief
 	* Used to add flags
	* @param f The flag(s) to add.
 	*/
	void operator+=(const flags &f) { _flags |= f; }
	
	/**@brief
 	* Used to remove flags
	* @param f The flag(s) to remove.
 	*/
	void operator-=(const flags &f) { _flags = _flags & (_flags ^ f); }
	
	private:
	u_int32_t _flags;
};

} //End namespace cppe

#endif
