#ifndef CPPE_OUTPUT_HPP
#define CPPE_OUTPUT_HPP

#include <string>

#include <cppe/output_base.hpp>
#include <cppe/stdout.hpp>

namespace cppe { 

/**@brief
 * Global output system. Use this class to output info
 * or error messages to the user/system.
 * The default setting is to output to standard output
 * and to not be verbose (only output error messages).
 */

class output {
	public:
	
	static output& inst();
    
	void verbose(bool ver) { _verbose = ver; }
	bool verbose() const { return _verbose; }
	void write(output_base::Type type, const std::string& message);
	void base(output_base *new_base);
	~output() { delete _out; }
	
	private:
	output_base* _out;
	bool _verbose;
	
	output() : _out(new stdout()),  _verbose(false) {}
};


inline void output::write(output_base::Type type, const std::string& message) {
	if(type == output_base::INFO && _verbose == false)
		return;
	_out->write(type, message);
}

inline output& output::inst() {
	static output myself;
	return myself;
}

inline void output::base(output_base *new_base) {
	delete _out;
	_out = new_base;
}

} //End namespace cppe

#endif
