#ifndef CPPE_PERMISSIONS_HPP
#define CPPE_PERMISSIONS_HPP

#include <string>

#include <sys/stat.h>	//mode_t

#include <cppe/cppe_exception.hpp>

namespace cppe {

/**@brief
 * The flags class provide easy handling of filsystem permissions.
 */

class permissions {
	public:
	class bad_permission_exception : public cppe_exception {
		public:
		bad_permission_exception(const std::string& m) : cppe_exception(m) {}
	};
	
	/**@brief 
	 * Constructor.
	 * @param perms numric presentation of the permissions (e.g "725").
	 * @throws bad_permission_exception if \c perms has an unknown permission.
	 */
	permissions(const std::string &perms);
	permissions(mode_t perms) : _perms(perms) {}
	
	operator mode_t() const { return _perms; }
	
	/**@brief
 	* Used to add permissions.
	* @param f The permission(s) to add.
 	*/
	void operator+=(const permissions &p) { _perms |= p; }
	
	/**@brief
 	* Used to remove permissions.
	* @param p The permission(s) to remove.
 	*/
	void operator-=(const permissions &p) { _perms = _perms & (_perms ^ p); }
	
	private:
	mode_t _perms;	
};

} //End namespace cppe

#endif
