#ifndef CPPE_PLUGIN_LOADER_HPP
#define CPPE_PLUGIN_LOADER_HPP

#include <string>
#include <map>

#include <dlfcn.h>	//dlopen, dlsym, dlclose

#include <cppe/plugin_exception.hpp>

namespace cppe {

/**@brief
 * plugin_loader can load any shared object that has
 * new_plugin and delete_plugin
 */

template <typename T> class plugin_loader {
	public:
	 T* plugin_loader::load_plugin(const std::string &file) {
		using namespace std;
		void* plugin_handler = dlopen(file.c_str(), RTLD_NOW);
		if(!plugin_handler) {
			string message = "Error loading: " + file;
			throw plugin_exception(message);
		}
		
		typedef T* create_plugin();
		typedef void destroy_plugin(T*);
		create_plugin* new_plugin = (create_plugin*) dlsym(plugin_handler, "new_plugin");
		destroy_plugin* delete_plugin = (destroy_plugin*) dlsym(plugin_handler, "delete_plugin");
	
		if (!new_plugin) {
			dlclose(plugin_handler);
			string message = "No 'new_plugin' in:" + file;
			throw plugin_exception(message);
		}
	
		if (!delete_plugin) {
			dlclose(plugin_handler);
			string message = "No 'delete_plugin' in:" + file;
			throw plugin_exception(message);
		}
	
		 T* plugin = new_plugin();
		_loaded_plugins[plugin] = plugin_handler;
		return plugin;
	}

	void plugin_loader::delete_plugin(T *plugin) {
		typename std::map<T*, void*>::iterator it = _loaded_plugins.find(plugin);
		if(it != _loaded_plugins.end()) {
			typedef void destroy_plugin(T*);
			destroy_plugin* delete_plugin = (destroy_plugin*) dlsym(it->second, "delete_plugin");
			delete_plugin(plugin);
			dlclose(it->second);
			_loaded_plugins.erase(it);
		}
	}

	private:
	/**Key = The plugin, value = handler*/
	std::map<T*, void*> _loaded_plugins;
};

} //End namespace cppe

#endif
