#ifndef CPPE_SLICE_HPP
#define CPPE_SLICE_HPP

#include <string>

#include <sys/param.h>  //statfs
#include <sys/mount.h>  //statfs

#include <cppe/cppe_exception.hpp>

namespace cppe {


class slice {
	class slice_exception : cppe_exception {
		public:
		slice_exception(const std::string& m) : cppe_exception(m) {}
	};
	
	public:
	slice(const std::string &slice_name) : _slice_name(slice_name) {
		update();
	}
	
	unsigned long size() const {
		return _slice.f_bsize * _slice.f_blocks;
	}
	unsigned long free() const {
		return _slice.f_bsize * _slice.f_bavail;
	}
	
	unsigned long used() const {
		return size() - free();
	}
	
	void update() {
		if(statfs(_slice_name.c_str(), &_slice) == -1)
			throw slice_exception(_slice_name);
	}
		
	private:
	std::string _slice_name;
	struct statfs _slice;
};

} //End namespace cppe

#endif



