#ifndef CPPE_STRING_HELPERS_HPP
#define CPPE_STRING_HELPERS_HPP

#include <string>
#include <sstream>

#include <ctype.h>	//isspace

namespace cppe {

/**@brief 
 * Find a non whitespace char in a string.
 * @param tmp The string to search in.
 * @param from Where in the string to start the search.
 * @return A pointer to the non-whitespace char.
 */
inline std::string::size_type find_non_ws(const std::string &tmp, std::string::size_type from = 0) {
	for(;from < tmp.size();++from){
		if(isspace(tmp[from]) == 0)
			return from;
	}
	return std::string::npos;
}

/**@brief 
 * Find a non whitespace char in a string, searching backwards.
 * @param tmp The string to search in.
 * @param from Where in the string to start the search.
 * @return A pointer to the non-whitespace char.
 */
inline std::string::size_type rfind_non_ws(const std::string &tmp, std::string::size_type from) {
	for(;from >= 0; --from){
		if(isspace(tmp[from]) == 0)
			return from;
	}
	return std::string::npos;
}

/**@brief 
 * Find a non whitespace char in a string, searching backwards.
 * @param tmp The string to search in.
 * @return A pointer to the non-whitespace char.
 */
inline std::string::size_type rfind_non_ws(const std::string &tmp) {
	return rfind_non_ws(tmp, tmp.size() - 1);
}

/**@brief 
 * Find a whitespace in a string.
 * @param tmp The string to search in.
 * @param from Where in the string to start the search.
 * @return A pointer to the whitespace.
 */
inline std::string::size_type find_ws(const std::string &tmp, std::string::size_type from = 0) {
	for(;from < tmp.size();++from){
		if(isspace(tmp[from]) != 0)
			return from;
	}
	return std::string::npos;
}

/**@brief 
 * Convert an int to a string.
 * @param i the integer to convert.
 * @return The string \c i converts too.
 */
inline std::string int_to_string(int i) {
	std::stringstream ss;
	ss<<i;
	return ss.str();
}

} //End namespace cppe

#endif
 
