#include <cppe/string_helpers.hpp>
#include <cppe/field.hpp>

namespace cppe {

void field::extend(unsigned int from, unsigned int too) {
	if(from == too || too < from)
		return;	
	field *f = _subfields[from];
	f->_end = _subfields[too]->_end;
	f->_size = f->_end + 1 - f->_begin;
	f->clear();
	std::vector<field*>::iterator begin = _subfields.begin();
	for(unsigned i = from + 1; i <= too; ++i)
		delete _subfields[i];
	_subfields.erase(begin + from+1, begin + too + 1);
}

void field::split() {
	_subfields.clear();
	ssize start = find_non_ws(_target, _begin);		//Find start of first field
	while(start != std::string::npos && start <= _end) {		//Start of a field found and within limits
		ssize end = find_ws(_target, start);		//Find end of field
		if(end == std::string::npos || end > _end) {		//Field end reached end of limits
			_subfields.push_back(new field(_target, start, 1+_end - start));
			break;
		}
		_subfields.push_back(new field(_target, start, end - start));
		start = find_non_ws(_target, end);		//Find start of next field
	}
}

void field::split(char separator) {
	_subfields.clear();
	ssize start = find_non_ws(_target, _begin);		//Find start of first field
	while(start != std::string::npos && start <= _end) {
		ssize pos = _target.find(separator, start);	//Find separator
		if(pos == std::string::npos || pos > _end) {		//Separator not found or not within limits
			ssize end =  rfind_non_ws(_target, _end);	//Find end of field
			_subfields.push_back(new field(_target, start, 1+end - start));
			break;
		}

		if(start == pos) {				//Empty field
			_subfields.push_back(new field(_target, start, 0));
		}
		else {
			ssize end =  rfind_non_ws(_target, pos - 1);	//Find end of field
			_subfields.push_back(new field(_target, start, 1+end - start));
		}
		start = find_non_ws(_target, pos + 1);		//Find start of next field
	}
}

}
