#include <string>
#include <vector>
#include <iostream>

#include <cppe/comment_filter.hpp>
#include <cppe/filter_iterator.hpp>
#include <cppe/field.hpp>
#include <cppe/file.hpp>
#include <cppe/output.hpp>

#include <editfile/conf_reader.hpp>

using namespace std;
using namespace cppe;
using namespace editfile;

int main(int argc, char *argv[]) {
	try {
	conf_reader &cf = conf_reader::inst();
	cf.process(argc, argv);
	comment_filter &filter = cf.comment_filter();
	filter_iterator fi(filter);
	file f(cf.file());
	vector<string> tmp;
	f.read(tmp);
	vector<string>::iterator it = tmp.begin();
	for(; fi.next(it, tmp.end()); ++it) {	//Find login class
		field ff(*it, fi.first(), fi.last() - fi.first() + 1);
		
		string::size_type sep = string::npos;
		if(cf.autodetect_separator()) {
			sep = (*it).find("=");
			if(sep != string::npos)
				ff.split('=');
			else
				ff.split();
		}
		else {
			if(cf.separator() != '\t') {
				ff.split(cf.separator());
				sep = (*it).find(cf.separator());
			}
			else
				ff.split();
		}

		if(!ff[0].compare(cf.keyword())) 
			continue;

		if(cf.remove()) {		//Remove the line?
			tmp.erase(it);
			f.write(tmp);
			return 0;
		}

		if(cf.comment_out()) {		//Comment out the line?
			*it = cf.comment() + *it;
			f.write(tmp);
			return 0;
		}

		if(ff.fields() > 2)
			ff.extend(1);

		if(ff.fields() == 2) {			//Keyword in file has a value
			if(cf.value().empty()) {	//User didn't give a value
				if(sep != string::npos)
					(*it).erase(++sep, ff[1].end() - (sep));
				else
					(*it).erase(ff[0].end() + 1, ff[1].end() - ff[0].end());
			}
			else	//Replace the value with the one the user specified
				(*it).replace(ff[1].begin(), ff[1].size(), cf.value());
			f.write(tmp);
			return 0;
		}
		
		if(!cf.value().empty() && ff.fields() == 1) {	//No value in file, but the user specified one
			if(sep != string::npos)
				(*it).insert(++sep, cf.value());
			else
				(*it).insert(ff[0].end() + 1, cf.separator() + cf.value());
			f.write(tmp);
			return 0;
		}
		return 0;	//No value given by user and no value in the file, so just quit
	}

	if(cf.add()) {		//No match was found, add the keyword?
		tmp.push_back(cf.keyword() + cf.separator() + cf.value());
		f.write(tmp);
		return 0;
	}
	} //End try
	catch(editfile_exception &e) {
		output::inst().write(output_base::ERROR, e.what() + "\n");
	}
	catch(file::not_found_exception &e) {
		output::inst().write(output_base::ERROR, "No such file: " + e.what() + "\n");
	}
	catch(cppe_exception &e) {
		output::inst().write(output_base::ERROR, "cppe_exception: " + e.what() + "\n");
	}
	catch(exception &e) {
		output::inst().write(output_base::ERROR, string(e.what()).append("\n"));
	}
	catch(...) {
		output::inst().write(output_base::ERROR, "Unknown exception\n");
	}
}
