#ifndef EDITFSTAB_CONF_READER_HPP
#define EDITFSTAB_CONF_READER_HPP

#include <cppe/para_reader.hpp>
#include <cppe/output.hpp>
#include <cppe/syslog.hpp>

#include <editfstab/editfstab_exception.hpp>

namespace editfstab {

class conf_reader {
	public:
	class conf_reader_exception : public editfstab_exception {
		public:
		conf_reader_exception(const std::string &message) : editfstab_exception(message) {}
	};

	bool add() const { return _add; }
	const std::string& file() const { return _file; }
	const std::string& filesystem() const { return _filesystem; }
	const std::string& options() const { return _options; }

	static conf_reader& inst() {
		static conf_reader myself;
		return myself;
	}

	void process(int argc, char *argv[]) {
		cppe::para_reader pr(argc, argv);
		process_q_flag(pr);
		process_file(pr);
		process_filesystem(pr);
		process_options(pr);
	}

	private:
	conf_reader() : _add(false), _file("/etc/fstab") {}
	void process_q_flag(cppe::para_reader &pr);
	void process_file(cppe::para_reader &pr);
	void process_filesystem(cppe::para_reader &pr);
	void process_options(cppe::para_reader &pr);
	bool _add;
	std::string _file;
	std::string _filesystem;
	std::string _options;
};

inline void conf_reader::process_q_flag(cppe::para_reader &pr) {
	if(pr.find_flag("q")) {
		cppe::output::inst().base(new cppe::syslog("editfstab"));
		cppe::output::inst().verbose(false);
	}
}

inline void conf_reader::process_file(cppe::para_reader &pr) {
	std::string file;
	if(pr.find_flag("f", file)) {
		if(file.empty()) 
			throw conf_reader_exception("No file specified");
		_file = file;
	}
}

inline void conf_reader::process_filesystem(cppe::para_reader &pr) {
	pr.flagless_value(_filesystem);
	if(_filesystem.empty()) 
		throw conf_reader_exception("No filesystem specified");
}

inline void conf_reader::process_options(cppe::para_reader &pr) {
	pr.flagless_value(_options);
	if(_options.empty())
		throw conf_reader_exception("No options specified");

	if(_options[0] == '+') {
		_add = true;
		_options.erase(0,1);
	}
}

} //End namespace editfstab

#endif
