#include <string>
#include <vector>
#include <set>

#include <cppe/comment_filter.hpp>
#include <cppe/filter_iterator.hpp>
#include <cppe/field.hpp>
#include <cppe/file.hpp>
#include <cppe/output.hpp>

#include <editfstab/conf_reader.hpp>
#include <editfstab/merge.hpp>

using namespace std;
using namespace cppe;
using namespace editfstab;

int main(int argc, char *argv[]) {
	try {
	conf_reader &cf = conf_reader::inst();
	cf.process(argc, argv);
	comment_filter filter;
	filter_iterator fi(filter);
	filter.add_comment_starter("#");
	file f(cf.file());
	vector<string> tmp;
	f.read(tmp);
	vector<string>::iterator it = tmp.begin();
	for(; fi.next(it, tmp.end()); ++it) {
		field file_options(*it, fi.first(), fi.last() - fi.first() + 1);
		file_options.split();
		if(file_options.fields() < 4) {
			output::inst().write(output_base::ERROR, "Syntax error: " + *it + "\n");
			return 0;
		}

		if(!(cf.filesystem() == file_options[0] || cf.filesystem() == file_options[1]))	//No match
			continue;

		if(!cf.add()) {			//Don' want to add
			(*it).replace(file_options[3].begin(), file_options[3].size(), cf.options());
			f.write(tmp);
			return 0;
		}
		vector<field*> final_options;

		field user_options(cf.options());
		merge_options(file_options[3], user_options, final_options);

		string new_options;
		
		for(unsigned int i = 0; i < final_options.size();) {
			new_options += final_options[i]->str();
			++i;
			if(i < final_options.size())
				new_options += ",";
		}
		(*it).replace(file_options[3].begin(), file_options[3].size(), new_options);
		f.write(tmp);
		return 0;
	}
	} //End try
	catch(editfstab_exception &e) {
		output::inst().write(output_base::ERROR, e.what() + "\n");
	}
	catch(file::not_found_exception &e) {
		output::inst().write(output_base::ERROR, "No such file: " + e.what() + "\n");
	}
	catch(cppe_exception &e) {
		output::inst().write(output_base::ERROR, "cppe_exception: " + e.what() + "\n");
	}
	catch(exception &e) {
		output::inst().write(output_base::ERROR, string(e.what()).append("\n"));
	}
	catch(...) {
		output::inst().write(output_base::ERROR, "Unknown exception\n");
	}
}
