#ifndef EDITKERNEL_CONF_READER_HPP
#define EDITKERNEL_CONF_READER_HPP

#include <cppe/comment_filter.hpp>
#include <cppe/field.hpp>
#include <cppe/para_reader.hpp>
#include <cppe/output.hpp>
#include <cppe/syslog.hpp>

#include <editkernel/editkernel_exception.hpp>

namespace editkernel {

class conf_reader {
	public:
	class conf_reader_exception : public editkernel_exception {
		public:
		conf_reader_exception(const std::string &message) : editkernel_exception(message) {}
	};

	const std::string& file() const { return _file; }
	const std::string& option() const { return _option; }
	bool hint() const { return _hint; }
	void process(int argc, char *argv[]);

	static conf_reader& inst() {
		static conf_reader myself;
		return myself;
	}

	private:
	conf_reader() : _hint(false) {}

	void process_file(cppe::para_reader &pr);	//Find the file to use.
	void process_option(cppe::para_reader &pr);	//Find the keyword (and value) to use.
	void process_q_flag(cppe::para_reader &pr);

	std::string _file;
	std::string _option;
	bool _hint;
};

inline void conf_reader::process(int argc, char *argv[]) {
	cppe::para_reader pr(argc, argv);
	process_q_flag(pr);
	process_file(pr);
	process_option(pr);
}

inline void conf_reader::process_q_flag(cppe::para_reader &pr) {
	if(pr.find_flag("q")) {
		cppe::output::inst().base(new cppe::syslog("editkernel"));
		cppe::output::inst().verbose(false);
	}
}

inline void conf_reader::process_file(cppe::para_reader &pr) {
	pr.flagless_value(_file);
	if(_file.empty())
		throw conf_reader_exception("No file specified");
}

inline void conf_reader::process_option(cppe::para_reader &pr) {
	std::vector<std::string> tmpv;
	pr.values_left(tmpv);
	if(tmpv.empty())
		throw conf_reader_exception("No option specified");

	if(tmpv[0].find('=') != std::string::npos)
		_hint = true;

	unsigned int i = 0;
	while(true) {
		_option += tmpv[i];
		++i;
		if(i < tmpv.size())
			_option += " ";
		else
			break;
	}
}

} //End namespace editkernel

#endif
