#ifndef EDITLOGIN_CONF_READER_HPP
#define EDITLOGIN_CONF_READER_HPP

#include <cppe/para_reader.hpp>
#include <cppe/output.hpp>
#include <cppe/syslog.hpp>

#include <editlogin/editlogin_exception.hpp>

namespace editlogin {

class conf_reader {
	public:
	class conf_reader_exception : public editlogin_exception {
		public:
		conf_reader_exception(const std::string &message) : editlogin_exception(message) {}
	};

	const std::string& file() const { return _file; }
	const std::string& login_class() const { return _class; }
	const std::string& option() const { return _option; }

	static conf_reader& inst() {
		static conf_reader myself;
		return myself;
	}

	void process(int argc, char *argv[]) {
		cppe::para_reader pr(argc, argv);
		process_q_flag(pr);
		process_file(pr);
		process_class(pr);
		process_option(pr);
	}

	private:
	conf_reader() : _file("/etc/login.conf") {}
	void process_q_flag(cppe::para_reader &pr);
	void process_file(cppe::para_reader &pr);
	void process_class(cppe::para_reader &pr);
	void process_option(cppe::para_reader &pr);
	std::string _file;
	std::string _class;
	std::string _option;
};

inline void conf_reader::process_q_flag(cppe::para_reader &pr) {
	if(pr.find_flag("q")) {
		cppe::output::inst().base(new cppe::syslog("editlogin"));
		cppe::output::inst().verbose(false);
	}
}

inline void conf_reader::process_file(cppe::para_reader &pr) {
	std::string file;
	if(pr.find_flag("f", file)) {
		if(file.empty()) 
			throw conf_reader_exception("No file specified");
		_file = file;
	}
}

inline void conf_reader::process_class(cppe::para_reader &pr) {
	pr.flagless_value(_class);
	if(_class.empty()) 
		throw conf_reader_exception("No class specified");
}

inline void conf_reader::process_option(cppe::para_reader &pr) {
	std::vector<std::string> tmpv;
	pr.values_left(tmpv);
	if(tmpv.empty())
		throw conf_reader_exception("No option specified");

	unsigned int i = 0;
	while(true) {
		_option += tmpv[i];
		++i;
		if(i < tmpv.size())
			_option += " ";
		else
			break;
	}
}

} //End namespace editlogin

#endif
