#ifndef EDITFSTAB_CONF_READER_HPP
#define EDITFSTAB_CONF_READER_HPP

#include <cppe/para_reader.hpp>
#include <cppe/output.hpp>
#include <cppe/syslog.hpp>

#include <editttys/editttys_exception.hpp>

namespace editttys {

class conf_reader {
	public:
	class conf_reader_exception : public editttys_exception {
		public:
		conf_reader_exception(const std::string &message) : editttys_exception(message) {}
	};

	const std::string& file() const { return _file; }
	const std::string& terminal() const { return _terminal; }
	const std::string& status() const { return _status; }

	static conf_reader& inst() {
		static conf_reader myself;
		return myself;
	}

	void process(int argc, char *argv[]) {
		cppe::para_reader pr(argc, argv);
		process_q_flag(pr);
		process_file(pr);
		process_terminal(pr);
		process_status(pr);
	}

	private:
	conf_reader() : _file("/etc/ttys") {}
	void process_q_flag(cppe::para_reader &pr);
	void process_file(cppe::para_reader &pr);
	void process_terminal(cppe::para_reader &pr);
	void process_status(cppe::para_reader &pr);
	std::string _file;
	std::string _terminal;
	std::string _status;
};

inline void conf_reader::process_q_flag(cppe::para_reader &pr) {
	if(pr.find_flag("q")) {
		cppe::output::inst().base(new cppe::syslog("editttys"));
		cppe::output::inst().verbose(false);
	}
}

inline void conf_reader::process_file(cppe::para_reader &pr) {
	std::string file;
	if(pr.find_flag("f", file)) {
		if(file.empty()) 
			throw conf_reader_exception("No file specified");
		_file = file;
	}
}

inline void conf_reader::process_terminal(cppe::para_reader &pr) {
	pr.flagless_value(_terminal);
	if(_terminal.empty())
		throw conf_reader_exception("No terminal specified");
}

inline void conf_reader::process_status(cppe::para_reader &pr) {
	pr.flagless_value(_status);
	if(_status.empty())
		throw conf_reader_exception("No status specified");
}

} //End namespace editttys

#endif
