#include <string>
#include <vector>
#include <iostream>

#include <cppe/comment_filter.hpp>
#include <cppe/filter_iterator.hpp>
#include <cppe/field.hpp>
#include <cppe/file.hpp>
#include <cppe/output.hpp>

#include <editttys/conf_reader.hpp>

using namespace std;
using namespace cppe;
using namespace editttys;

int main(int argc, char *argv[]) {
	try {
	conf_reader &cf = conf_reader::inst();
	cf.process(argc, argv);
	bool console = false;
	bool secure = false;
	if(cf.terminal() == "console")
		console = true;
	else if(cf.terminal() != "tty") {
		output::inst().write(output_base::ERROR, "Terminal must be 'console' or 'tty'\n");
		return 0;
	}
	
	if(cf.status() == "secure")
		secure = true;
	else if(cf.status() != "insecure") {
		output::inst().write(output_base::ERROR, "Status must be 'secure' or 'insecure'\n");
		return 0;
	}

	comment_filter filter;
	filter.add_comment_starter("#");
	filter_iterator fi(filter);
	file f(cf.file());
	vector<string> tmp;
	f.read(tmp);
	vector<string>::iterator it = tmp.begin();
	for(; fi.next(it,tmp.end()); ++it) {

		field option(*it, fi.first(), fi.last() - fi.first() + 1);
		option.split();
		if(option.fields() < 4)
			continue;
		if((*it)[option[1].begin()] == '\"') {
			bool found = false;
			for(unsigned int i = 2; i < option.fields(); ++i) {
				if((*it)[option[i].end()] == '\"') {
					option.extend(1, i);
					found = true;
					break;
				}
			}
			if(!found) {
				output::inst().write(output_base::ERROR, "Syntax error: '" + *it + "' no endning \" found" + "\n");
				return 0;
			}
		}
		if(option.fields() < 4)
			continue;

		unsigned int status = 0;
		for(unsigned int i = 3; i < option.fields(); ++i) {
			if(option[i] == "secure" || option[i] == "insecure") {
				status = i;
				break;
			}
		}

		if(status == 0)
			continue;

		if(console) {
			if(option[0] != "console")
				continue;
			if(secure)
				(*it).replace(option[status].begin(), option[status].size(), "secure");
			else
				(*it).replace(option[status].begin(), option[status].size(), "insecure");
			f.write(tmp);
			return 0;
		}
		
		if(option[0].str().find("tty") == option[0].begin()) {
			if(secure)
				(*it).replace(option[status].begin(), option[status].size(), "secure");
			else
				(*it).replace(option[status].begin(), option[status].size(), "insecure");
		}
		f.write(tmp);
	}
	}
	catch(editttys_exception &e) {
		output::inst().write(output_base::ERROR, e.what() + "\n");
	}
	catch(file::not_found_exception &e) {
		output::inst().write(output_base::ERROR, "No such file: " + e.what() + "\n");
	}
	catch(cppe_exception &e) {
		output::inst().write(output_base::ERROR, "cppe_exception: " + e.what() + "\n");
	}
	catch(exception &e) {
		output::inst().write(output_base::ERROR, string(e.what()).append("\n"));
	}
	catch(...) {
		output::inst().write(output_base::ERROR, "Unknown exception\n");
	}
}
